/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup;

import java.io.*;


/**
 * The <code>DistributedSystem</code> class is used to describe the distributed
 * system in which the computation is carried on. How a distributed system is
 * described depends on the specific implementation; for example, a distributed
 * system may be described by a single multicast address, or a set of unicast
 * addresses, or a combination of them. Here, we simply assume that the
 * distributed system description is contained in a configuration file.
 * The <code>DistributedSystem</code> instances store the pathnames of these
 * configuration files.
 *
 * @author  Alberto Montresor
 * @author  Hein Meling
 * @since   Jgroup 0.8
 */
public class      DistributedSystem
implements        Serializable
{

  /////////////////////////////////////////////////////////////////////////////
  // Fields
  /////////////////////////////////////////////////////////////////////////////

  /*
   * Pathname of the file describing the distributed system
   */
  private String      configfile;
  private String      configURL;

  /////////////////////////////////////////////////////////////////////////////
  // Constructors
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Disable default constructor.
   */
  private DistributedSystem()
  {
  }

  /**
   * Instantiates a distributed system whose description is
   * contained in <code>configfile</code>.
   *
   * @param configfile    The pathname of the file containing the
   *                      description of the distributed system.
   */
  public DistributedSystem(String configfile)
  {
    this.configfile = configfile;
    // NB: This should change later...
    this.configURL = configfile;
  }

  /////////////////////////////////////////////////////////////////////////////
  // Public methods
  /////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the path to the config file.
   */
  public String getConfigfile()
  {
    return configfile;
  }

  /**
   *  Returns the path to the config file.
   */
  public String getConfigURL()
  {
    return configURL;
  }

} // END  DistributedSystem
