/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup;

import java.rmi.*;
import java.io.*;

import jgroup.registry.*;

/**
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.8
 */
public interface  ExternalGMIService
{

  /**
   *
   */
  public IID getIdentifier();


  /**
   * Bind the server associated to this group manager to the object group associated
   * with the specified name in the dependable registry running in the same distributed
   * system as this group manager.
   *
   * @param name        the name associated with the object group
   * @return            a binding identifier <tt> IID</tt> used to remove this
   *                    binding from the registry
   *
   * @exception RemoteException raised when remote operation failed.
   * @exception AccessException raised when this operation is not permitted.
   */
  public IID bind(String name)
    throws RemoteException, AccessException;


  /**
   * Bind the server associated to this group manager to the object group associated
   * with the specified name in the specified dependable registry.
   *
   * @param name        the name associated with the object group
   * @return            a binding identifier <tt> IID</tt> used to remove this
   *                    binding from the registry
   *
   * @exception RemoteException raised when remote operation failed.
   * @exception AccessException raised when this operation is not permitted.
   */
  public IID bind(String name, DependableRegistry registry)
    throws RemoteException, AccessException;

} // END ExternalGMIService

