/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup;

/**
 * The <code> TransportConfig</code> class contains the basic information
 * used to configure the transport layer of the Jgroup implementation.
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.8
 */
public class    TransportConfig
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Public constants
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Default Jgroup port */
  public static final int JGROUP_PORT           = 28771;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Port used by the transport layer to communicate */
  private int           port;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs a <code> TransportConfig</code> with the default Jgroup port.
   *
   */
  public TransportConfig()
  {
    this.port = JGROUP_PORT;
  }

  /**
   * Constructs a <code> TransportConfig</code> with the specified port.
   *
   * @param port  The port to be used
   */
  public TransportConfig(int port)
  {
    if (port != 0) 
      this.port = port;
    else
      this.port = JGROUP_PORT;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Return the port number stored in this object.
   *
   */
  public int getPort()
  {
    return port;
  }

} // END TransportConfig
