/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.exec;

import java.util.*;

/**
 * The <code>DistributionScheme</code> interface provides a standard
 * for implementing various distribution schemes that simplify interaction
 * with the <code>ReplicaManager</code>.
 *
 * @author  Hein Meling
 * @since   Jgroup 1.1
 */
public interface DistributionScheme {

  /////////////////////////////////////////////////////////////////////////////
  // Constants
  /////////////////////////////////////////////////////////////////////////////

  /////////////////////////////////////////////////////////////////////////////
  // Methods
  /////////////////////////////////////////////////////////////////////////////

  /**
   *  Updates the internal tables based on a list of suspected hosts.
   *
   *  @param suspect_hosts  List of suspected hosts to check for availability.
   *
   */
  public void suspect(ArrayList suspect_hosts);

  /**
   * The <code>getExecRef</code> method returns the remote reference for
   * the specified host.
   *
   * @return          host name
   *
   */
  public ExecService getExecRef(String host)
    throws HostException;

  /**
   * The <code>getHost</code> method will return the least used host.
   *
   * @return          host name
   *
   */
  public String getHost()
    throws HostException;

  /**
   * The <code>getHosts</code> method will return a set of hosts that
   * preferably occur in different domains, and that are the least used
   * hosts within each domain.
   *
   * @return          set of host names
   *
   */
  public String[] getHosts(int redundancy)
    throws RedundancyException;

  /**
   * The <code>getHosts</code> method will return two sets of hosts that
   * preferably occur in different domains, and that are the least used
   * hosts within each domain, and that preferably don't overlap each
   * other. Can be used to get non-overlapping hosts for two groups,
   * for instance, one group with ReplicaManager replicas, and the other
   * with Dependable Registry replicas.
   *
   * @return          set of host names [groupA, groupB]. The first part
   *                  of the returned array belongs to groupA, while the
   *                  second part belongs to groupB.
   *
   */
  public String[] getHosts(int redundancyA, int redundancyB);

} // END DistributionScheme
