/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.exec;

import java.rmi.*;

/**
 * The <code>ExecService</code> interface describes the methods
 * of an execution service that can be remotely invoked.
 *
 * @author  Hein Meling
 * @since   Jgroup 1.1
 */
public interface ExecService
extends Remote
{

  /////////////////////////////////////////////////////////////////////////////
  // Constants
  /////////////////////////////////////////////////////////////////////////////

  /*
   * Port number used by the exec daemons for bootstrap purposes.
   * Each exec daemon has its own local standard RMI registry using this port.
   *
   */
  public static final int     EXEC_BOOTSTRAP_PORT = 1201;

  /*
   * Service name used to register each exec daemon in its local registry.
   *
   */
  public static final String  EXEC_DAEMON_NAME = "ExecDaemon";

  /////////////////////////////////////////////////////////////////////////////
  // Methods
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Instantiate and start the given class.
   *
   * @param c         the class representing the replica.
   * @param args      arguments to the replica.
   * @return          identifier for the instantiated class.
   *
   * @exception RemoteException If remote operation failed.
   * @exception Exception If the class could not be instantiated.
   */
  public void start(Class c, String[] args)
    throws RemoteException, ExecException;

  /**
   * Instantiate and start the given class in a new JVM process.
   *
   * @param classname  the class representing the replica.
   * @param args       arguments to the replica.
   * @return           identifier for the instantiated class.
   *
   * @exception RemoteException  If remote operation failed.
   * @exception ExecException    If the specified class could not be instantiated
   *                             or is already running within this execution service.
   */
  public void createReplica(ClassData classData)
    throws RemoteException, ExecException;

  /**
   * Remove the local replica belonging to the specified group.
   *
   * @param groupid  Group identifier
   *
   * @exception RemoteException  If remote operation failed.
   * @exception ExecException    If the specified group has no local replica
   *                             running within this execution service.
   */
  public void removeReplica(int gid)
    throws RemoteException, ExecException;

} // END ExecService
