/*
 * Copyright (c) 1998-2001 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
package jgroup.exec;

/**
 * RecoveryStyle specifies the recovery style of a replica group.
 *
 * Note: The recovery style constants are subject to change.
 *
 * @author      Hein Meling
 * @since       Jgroup 1.1
 */
public interface RecoveryStyle {

  /////////////////////////////////////////////////////////////////////////////
  // Recovery style constants
  /////////////////////////////////////////////////////////////////////////////

  /**
   *  Don't recover from failed replicas.
   */
  public final static int  NO_RECOVERY               = 0;

  /**
   *  Keep a minimal number replicas in each partition.
   */
  public final static int  KEEP_MINIMAL_IN_PARTITION = 1;

  /**
   *  Lower and upper bound on the recovery style. Used for checking
   *  if the specified recovery style is valid. These should reflect
   *  the lowest and highest number in the list above.
   */
  public final static int LOWER_BOUND = NO_RECOVERY;
  public final static int UPPER_BOUND = KEEP_MINIMAL_IN_PARTITION;

  /**
   *  Lower and upper bound on the redundancy level allowed. Used for checking
   *  if the specified redundancy levels are valid.
   */
  public final static int REDUNDANCY_LOWER_BOUND = 1;
  public final static int REDUNDANCY_UPPER_BOUND = 9;

  /**
   *  Naming the recovery styles for printing.
   *  The ordering must correspond to the constants above.
   */
  public final static String[] RecoveryStyleNames = {
    "No recovery",
    "Keep minimal redundancy in each partition"
  };

  /////////////////////////////////////////////////////////////////////////////
  // Interface methods
  /////////////////////////////////////////////////////////////////////////////

  public int getRecoveryStyle();

  public void setRecoveryStyle(int recoveryStyle)
    throws Exception; // Was RecoveryException; problem in compiling; Alberto 3/5/01

  public int getInitialRedundancy();

  public void setInitialRedundancy(int initialRedundancy)
    throws Exception; // Was RecoveryException; problem in compiling; Alberto 3/5/01

  public int getMinimalRedundancy();

  public void setMinimalRedundancy(int minimalRedundancy)
    throws Exception; // Was RecoveryException; problem in compiling; Alberto 3/5/01
}
