/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package jgroup.exec;

import java.rmi.*;
import jgroup.*;

/**
 * The <code>ReplicationManager</code> interface describes the methods
 * of an execution service that can be remotely invoked.
 *
 * @author  Hein Meling
 * @since   Jgroup 1.1
 */
public interface ReplicationManager
extends ExternalGMIListener
{

  /////////////////////////////////////////////////////////////////////////////
  // Constants
  /////////////////////////////////////////////////////////////////////////////

  /*
   * Service name used to register the replication manager in the
   * dependable registry.
   *
   */
  public static final int     RMGID = 32000;

  /*
   * Service name used to register the replication manager in the
   * dependable registry.
   *
   */
  public static final String  REPLICATION_MANAGER_NAME = "ReplicationManager";

  /////////////////////////////////////////////////////////////////////////////
  // Methods
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Let ExecDaemons notify its presence to the Replication Manager.
   *
   * @param host      host running the ExecDaemon
   * @param ref       reference to the ExecDaemon
   * @return          returns false if host already registered
   *
   * @exception RemoteException If remote operation failed.
   * @exception McastRemoteException 
   */
  public boolean notifyPresence(String host, ExecService ref)
    throws RemoteException, McastRemoteException;

} // END ReplicationManager
