/*
 * Copyright (c) 1998-2001 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
package jgroup.exec;

/**
 * ReplicationStyle specifies the replication style of a replica.
 *
 * Note: The replication style constants are subject to change.
 *
 * @author      Hein Meling
 * @since       Jgroup 1.1
 */
public interface ReplicationStyle {

  /////////////////////////////////////////////////////////////////////////////
  // Replication style constants
  /////////////////////////////////////////////////////////////////////////////

  /**
   *  Stateless replication style;
   *  replicas have no state to synchronize.
   */
  public final static int  STATELESS = 0;

  /**
   *  Passive replication style (primary backup or standby redundancy);
   *  only the primary replica interacts with clients, but keeps the
   *  backup replicas synchronized with the primary.
   */
  public final static int  PASSIVE   = 1;

  /**
   *  Active replication style (modular redundancy);
   *  all replicas are synchronized.
   */
  public final static int  ACTIVE    = 2;

  /**
   *  Lower and upper bound on the replication style. Used for checking
   *  if the specified replication style is valid. These should reflect
   *  the lowest and highest number in the list above.
   */
  public final static int LOWER_BOUND = STATELESS;
  public final static int UPPER_BOUND = ACTIVE;

  /**
   *  Lower and upper bound on the redundancy level allowed. Used for checking
   *  if the specified redundancy levels are valid.
   */
  public final static int REDUNDANCY_LOWER_BOUND = 1;
  public final static int REDUNDANCY_UPPER_BOUND = 9;

  /**
   *  Naming the replication styles for printing.
   *  The ordering must correspond to the constants above.
   */
  public final static String[] ReplicationStyleNames = {
    "Stateless",
    "Passive replication",
    "Active replication"
  };

  /////////////////////////////////////////////////////////////////////////////
  // Interface methods
  /////////////////////////////////////////////////////////////////////////////

  public int getReplicationStyle();

  public void setReplicationStyle(int replicationStyle)
    throws ReplicationException;

  public int getInitialRedundancy();

  public void setInitialRedundancy(int initialRedundancy)
    throws ReplicationException;

  public int getMinimalRedundancy();

  public void setMinimalRedundancy(int minimalRedundancy)
    throws ReplicationException;
}
