/*
 * Copyright (c) 1998-2001 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.config;

import java.util.*;

/**
 * Class for holding registry information.
 *
 * @author Hein Meling
 * @since Jgroup 1.1
 */
public class RegistryConfig {

  /////////////////////////////////////////////////////////////////////////////
  // Default values
  /////////////////////////////////////////////////////////////////////////////

  private static final int  DREGISTRY_PORT    = 5672;
  private static final int  DREGISTRY_INITIAL = 2;
  private static final int  DREGISTRY_MINIMAL = 1;
  
  /////////////////////////////////////////////////////////////////////////////
  // Fields
  /////////////////////////////////////////////////////////////////////////////

  private int  regPort;      // Registry port
  private int  regInitial;   // Initial number of registry replicas
  private int  regMinimal;   // Minimal number of registry replicas

  /////////////////////////////////////////////////////////////////////////////
  // Constructors
  /////////////////////////////////////////////////////////////////////////////

  public RegistryConfig() {
    // Use default values
    this(DREGISTRY_PORT, DREGISTRY_INITIAL, DREGISTRY_MINIMAL);
  }

  public RegistryConfig(int port) {
    // Set inital and minimal to default.
    this(port, DREGISTRY_INITIAL, DREGISTRY_MINIMAL);
  }

  public RegistryConfig(int port, int initial, int minimal) {
    this.regPort    = port;
    this.regInitial = initial;
    this.regMinimal = minimal;
  }

  /////////////////////////////////////////////////////////////////////////////
  // Public RegistryConfig interface methods
  /////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the port number for the dependable registry service.
   */
  public int getPort() {
    return regPort;
  }

  /**
   *  Returns the initial number of dependable registry replicas.
   */
  public int getInitial() {
    return regInitial;
  }

  /**
   *  Returns the minimal number of dependable registry replicas.
   */
  public int getMinimal() {
    return regMinimal;
  }

  /////////////////////////////////////////////////////////////////////////////
  // Override methods in Object
  /////////////////////////////////////////////////////////////////////////////

  /**
   * Returns a string representation of the RegistryConfig object.
   */
  public String toString() {
    return "[" + regPort + ", " + regInitial + ", " + regMinimal + "]";
  }

} // END of class RegistryConfig
