/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.daemon;

import relacs.general.OrderedList;
import relacs.general.Queue;
import relacs.mss.*;

/**
 * The <code> HostData</code> class
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.1
 */
class           HostData
extends         OrderedList
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  // General fields
  Host                  host;                   // Host to which this information is related
  MsgProp               ctbl;                   // Coordinator table
  boolean               leaving;                // If the host is leaving the group
  int                   viewIndex;              // Index inside a view

  // Version numbers
  int                   version;                // Version number
  int                   agreed;                 // Agreed version number
  int                   last;                   // Version n. when last view has been installed

  // Fields related to messages
  int[]                 hack;                   // Hosts acknowledgement
  int[]                 mack;                   // Members acknowledgement
  int                   hmin;                   // Minimal value among host acknowledgement
  int                   delivered;              // Messages sent to members
  Queue                 buffer;                 // Message buffer


  // Fields associated with synchronization phase
  boolean               inSynchronized;         // True if it belongs to synchronized set
  boolean               inEstimate;             // True if it belongs to estimate set


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /*
   * Creates an empty HostData list.
   */
  HostData()
  {
    super();
  }


  /*
   * Creates a new additional info object for host <host>.
   */
  HostData(Host host)
  {
    super();

    key         = host.getAddress();
    this.host   = host;
    viewIndex   = 0;
    last        = 0;
    version     = 0;
    agreed      = 0;
    leaving     = false;
    ctbl        = null;
    hack        = new int[1];
    mack        = new int[1];
    hmin        = 0;
    delivered   = 0;
    buffer      = new Queue();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  HostData[] extract(MssHost[] list)
  {
    HostData      scan;               // Used to scan this HostData list
    int           i;                  // Used to scan <list>
    int           count;              // Number of pairs to be extracted

    count = 0;
    scan  = (HostData) this.next;
    i     = 0;
    while (scan != null && i< list.length) {
      if (scan.key == list[i].getAddress()) {
        count++;
        scan = (HostData) scan.next;
        i++;
      }
      else if (scan.key < list[i].getAddress())
        scan = (HostData) scan.getNext();
      else
        i++;
    }
    HostData[] temp = new HostData[count];
    count = 0;
    scan  = (HostData) this.next;
    i     = 0;
    while (scan != null && i< list.length) {
      if (scan.key == list[i].getAddress()) {
        temp[count++] = scan;
        scan = (HostData) scan.next;
        i++;
      }
      else if (scan.key < list[i].getAddress())
        scan = (HostData) scan.getNext();
      else
        i++;
    }
    return temp;
  }

  void newIncarnation()
  {
    version = 0;
    agreed  = 0;
    last    = 0;
  }

  /*
   *
   */
  void checkAck()
  {
    int min = Integer.MAX_VALUE;
    for (int i=0; i < hack.length; i++)
      if (min > hack[i])
        min = hack[i];
    if (min > hmin) {
      for (int i=hmin; i < min; i++)
        buffer.removeFirst();
      hmin = min;
    }
  }

} // END HostData

