/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.daemon;

import jgroup.*;
import jgroup.multicast.*;

import relacs.general.*;

/**
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.8
 */
final class SendBuffer
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  Queue       buffer;
  Queue       ackbuffer;
  ViewImpl    view;
  long        vid;
  int         hlen;
  int         nmessages;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  SendBuffer()
  {
    buffer = new Queue();
    ackbuffer = new Queue();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *
   */
  void setView(ViewImpl view, int hlen)
  {
    MsgResult   msgr;
    int         size;

    this.view = view;
    this.hlen = hlen;
    vid = view.getVid();
    nmessages = 0;

    // Call nack
    Object[] messages = buffer.toArray();
    for (int i=0; i < messages.length; i++) {
      AckListener ackl = ((MsgMcast) messages[i]).ackl;
      if (ackl != null) 
        ackl.viewChange();
    }

    buffer.clear();
    while ((msgr = (MsgResult) ackbuffer.removeFirst()) != null) {
      if (msgr.vid == vid)
        insertMsgResult(msgr);
    }
  }


  /**
   *
   */
  void insertMsgMcast(MsgMcast msg)
  {
    if (msg.ackl != null)
      msg.ackl.notifyView(view);
    buffer.insert(msg);
    msg.setAckArray(hlen);
  }


  /**
   *
   */
  void insertMsgResult(MsgResult result)
  {
    Object[]    messages;
    MsgMcast    message;

    if (result.vid == vid) {
      messages = buffer.toArray();
      for (int i=0; i < messages.length; i++) {
        message = (MsgMcast) messages[i];
        if (message.mid <= result.dlvr)
          message.setAck(result.hpos);
        if (message.mid == result.mid && message.sid == result.sid) {
          try {
            MemberId[] members = view.getMembers();
            if (message.ackl != null)
              message.ackl.ack(members[result.vpos], result.vpos, result.result);
          } catch (Exception e) { /* ??? */ }
        }
      }
      removeAcknowledged();
    } else {
      ackbuffer.insert(result);
    }
  }

  void suspect(int hpos)
  {
    Object[]    msgs;
    MsgMcast    msgm;

    msgs = buffer.toArray();
    for (int i=0; i < msgs.length; i++) {
      msgm = (MsgMcast) msgs[i];
      msgm.setAck(hpos);
    }
    removeAcknowledged();
  }


  /**
   * Remove messages that have been already acknowledged by every other host
   */
  private void removeAcknowledged()
  {
    MsgMcast    message;

    while ((message = (MsgMcast) buffer.getFirst()) != null) {
      if (message.isComplete())
        buffer.removeFirst();
      else
        break;
    }
  }


  int getMid()
  {
    return ++nmessages;
  }

  boolean isEmpty()
  {
    return buffer.isEmpty();
  }


} // END SendBuffer
