/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 *  History:
 *  22/11/2000: Added field sid, used to distinguish messages with
 *    the same id but originated by different senders.
 */

package relacs.daemon;

import jgroup.multicast.AckListener;
import relacs.mss.*;

/**
 * The <code> UpcallMcast</code> class represents a local message exchanged between
 * members and the daemon; it is used by the member to notify the daemon of its
 * willingness of sending a message, enclosed in the stream field of this class.
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.1
 */
final class     UpcallMcast
extends         Upcall
implements      Tag
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Message codified as message stream */
  OutMessage    stream;
  
  /** True if the stream contains a single object */
  boolean       flag;
  
  /** Identifier of the sender (see getLongIdentifier() of MemberIdImpl */
  long          sid;
  
  /** Ack listener to be notified */
  AckListener   ackl;

  
  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Creates a <code> UpcallMcast</code> upcall.
   */
  UpcallMcast(int gid, boolean flag, OutMessage stream, long sid, AckListener ackl)
  {
    super(L_MCAST, gid);
    this.stream = stream;
    this.flag = flag;
    this.sid = sid;
    this.ackl   = ackl;
  }

} // END UpcallMcast

