/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.general;

/**
 * The <code> OrderedList</code> class represents an ordered collections of objects.
 * The order of elements is based on their keys, implemented by an integer
 * value.
 *
 * see relacs.mss.MssMessage
 * see relacs.daemon.GidList
 * see relacs.daemon.HostData
 * see relacs.daemon.MemberData
 * see relacs.daemon.VersionList
 * see relacs.daemon.Group
 *
 * @author  Alberto Montresor
 * @since   JavaGroup 1.0
 */
public class OrderedList {

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** The identifier of this element */
  public int            key;

  /** Reference to the next element */
  protected OrderedList next;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Creates an empty list.
   */
  protected OrderedList()
  {
    next = null;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Inserts the specified element in the OrderedList. If the object's key is already present
   * in the OrderedList, the element is not inserted.
   *
   * @param object      the element to be inserted in this OrderedList.
   * @return            true if the object has been inserted; false otherwise.
   */
  public boolean insert(OrderedList object)
  {
    OrderedList                s;              // Used to scan list

    for (s = this; s.next != null && s.next.key < object.key; s = s.next);
    if (s.next == null || s.next.key > object.key) {
      object.next = s.next;
      s.next = object;
      return true;
    }
    return false;
  }

  /**
   * Removes the first occurrence of an  element with the specified key in this OrderedList.
   * If the OrderedList does not contain the element, it is unchanged and the methods returns
   * <code> null</code>.
   *
   * @param key         the key to search for.
   */
  public OrderedList remove(int key)
  {
    OrderedList                s;              // Used to scan list
    OrderedList                ret;            // Return value

    for (s = this; s.next != null && s.next.key < key; s = s.next);
    ret = null;
    if (s.next != null && s.next.key == key) {
      ret = s.next;
      s.next = s.next.next;
    }

    return ret;
  }

  /**
   *
   * @param key         the key to search for.
   */
  public void removeAll(int key)
  {
    OrderedList s;
    for (s = this; s.next != null && s.next.key <= key; s = s.next);
    this.next = s.next;
  }

  /**
   * Removes the first element of the list.
   */
  public OrderedList removeFirst()
  {
    OrderedList                ret;            // Return value

    ret = this.next;
    try { this.next = this.next.next; } catch (Exception e) { }
    return ret;
  }

  /**
   * Returns the first occurrence in this OrderedList of a element with the specified
   * key, or <code> null</code> if the element is not found.
   *
   * @param key         the key to search for.
   */
  public OrderedList lookup(int key)
  {
    OrderedList                s;              // Used to scan list

    for (s = this.next; s != null && s.key < key; s = s.next);
    return ((s != null && s.key == key) ? s : null);
  }

  /**
   * Returns the next element in the OrderedList.
   */
  public OrderedList getNext()
  {
    return next;
  }

  /**
   * Returns the first element of the OrderedList
   */
  public OrderedList getFirst()
  {
    return next;
  }

  /**
   * Returns true if this OrderedList contains no elements.
   */
  public boolean isEmpty()
  {
    return (next == null);
  }

  /**
   * Count the number of elements in the list
   */
  public int count()
  {
    int count = 0;
    OrderedList scan = next;
    while (scan != null) {
      count++;
      scan = scan.next;
    }
    return count;
  }


} // END OrderedList
