/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.general;

import debug.Debug;

/**
 * The <code> Util</code> class implements a collection of useful methods.
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.1
 */
public final class Util {

  /**
   * Private constructor (Util cannot have instance objects)
   */
  private Util() { }

  /**
   * Prints a panic message and exits.
   *
   * @param message             the error message.
   * @param e                   the exception that caused panic.
   */
  public static void panic(String message, Exception e)
  {
    Debug.println("Panic: "+message);
    e.printStackTrace(System.out);
    System.exit(-1);
  }

  /**
   * Prints a panic message and exits.
   *
   * @param message             the error message.
   */
  public static void panic(String message)
  {
    Debug.println("Panic: " + message);
    System.exit(-1);
  }

  /**
   * Searches the specified array of ints for the specified value using
   * the sequential search algorithm. The array <strong>must</strong>
   * be sorted prior to making this call.
   *
   * @param array       the array to be searched.
   * @param key         the value to be searched for.
   * @return            index of the search key, if it is contained in the array;
   *                    otherwise, array.length.
   */
  public static int valueAt(int[] array, int key)
  {
    int         i;                      // Counter

    for (i=0; i < array.length && array[i] < key; i++);
    return ((i < array.length && array[i] == key) ? i : -1);
  }

  public static int valueAt(Object[] array, Object key)
  {
    int         i;                      // Counter

    try {
      for(i=0; !array[i].equals(key); i++);
      return i;
    } catch (Exception e) {}
    return -1;
  }

  public static int referenceAt(Object[] array, Object key)
  {
    int         i;                      // Counter

    try {
      for(i=0; array[i] != key; i++);
      return i;
    } catch (Exception e) {}
    return -1;
  }

  public static boolean in(Object[] array, Object key)
  {
    int         i;                      // Counter

    try {
      for(i=0; array[i] != key; i++);
      return true;
    } catch (Exception e) {}
    return false;
  }

} // END Util

