/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.gm;

/**
 * The <code> SynchronizationLayer</code> class
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.9
 */
public class        SynchronizationLayer
implements          SynchronizationService
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  int       executing;      // Number of threads that have requested to disable events
  int       disabling;      // Number of threads that are executing events
  boolean   waiting;        // True if there threads waiting to execute events

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Initialize a new <tt>BaseLayer</tt> object using the informations contained in the
   * configuration objects.
   *
   * @param ds          Description of the distributed system
   * @param conf        Configuration parameters for the daemon used
   *                    by the group manager.
   *
   * @exception DSException
   *            raised when the distributed system configuration object contains an error
   * @exception IOException
   *            if an I/O error occurs when creating communication sockets.
   */
  private SynchronizationLayer(Object[] configuration)
  {
    executing = 0;
    disabling = 0;
    waiting = false;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static factory
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static SynchronizationLayer getLayer(Object[] configuration)
  {
    return new SynchronizationLayer(configuration);
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from SynchronizationService
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *
   */
  public synchronized void disableEvents()
  {
    while (executing > 0 && !waiting) {
      try { wait(); } catch (Exception e) { }
    }
    disabling++;
  }

  /**
   *
   */
  public synchronized void enableEvents()
  {
    disabling--;
    notifyAll();
  }


  /**
   *
   */
  public synchronized void startEvent()
  {
    waiting = true;
    while (disabling > 0) {
      try { wait(); } catch (Exception e) { }
    }
    waiting = false;
    executing++;
  }

  /**
   *
   */
  public synchronized void stopEvent()
  {
    executing--;
    notifyAll();
  }



} // END SynchronizationLayer



