/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.mss;

/**
 * The <code> EventList</code> class
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.1
 */
class EventList {


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  long                  value;          // Order relation is based on value
  int                   tag;            // Type of event
  Object                obj;            // Event descriptor

  EventList             next;           // Next element
  EventList             prev;           // Previous element



  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Constructs an empty ordered list.
   */
  EventList()
  {
  }

  /**
   *
   */
  EventList(int tag, Object obj)
  {
    this.tag = tag;
    this.obj = obj;
    this.next = null;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Inserts a new pair (value, )
   */
  void insert(long value, EventList event)
  {
    EventList   scan;           // Used to scan the list

    scan = this;
    while (scan.next != null && scan.next.value < value) {
      scan = scan.next;
    }
    event.value = value;
    event.prev = scan;
    event.next = scan.next;
    scan.next = event;
    try { event.next.prev = event; } catch (Exception e) { }
  }

  /**
   * Remove an event from the event list
   */
  void remove()
  {
    this.prev.next = this.next;
    try { this.next.prev = this.prev; } catch (Exception e) {  }
  }

  /**
   *  Returns the first value of the list
   */
  long getFirst()
  {
    return (this.next != null ? this.next.value : -1);
  }

  /**
   * Removes the first element of the list and returns the corresponding object.
   */
  EventList removeFirst()
  {
    EventList ret;                      // Return value

    ret = this.next;
    try {
      this.next = this.next.next;
      this.next.prev = this;
    } catch (Exception e) { }
    return ret;
  }

} // END EventList



