/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.mss;

import relacs.general.OrderedList;

/**
 * The <code> MssMessage </code> class
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.1
 */
class           MssMessage
extends         OrderedList
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  byte[]                buffer;
  int                   blen;
  MssHost               dest;
  EventList             event;
  long                  start;
  byte                  tag;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs an empty list of messages.
   */
  MssMessage()
  {
  }

  /**
   * Constructs and stores a sent message.
   */
  MssMessage(int mid, byte[] buffer, int blen, MssHost dest, long start, int tag)
  {
    this.key    = mid;
    this.buffer = buffer;
    this.blen   = blen;
    this.dest   = dest;
    this.event  = new EventList(tag, this);
    this.start  = start;
  }

  /**
   * Constructs and stores a received message.
   */
  MssMessage(int mid, byte[] buffer, int blen, byte tag)
  {
    this.key    = mid;
    this.buffer = buffer;
    this.blen   = blen;
    this.tag    = tag;
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Removes all the messages in the message list. Removes also the timeout.
   */
  void clear()
  {
    while (this.next != null) {
      try {
        ((MssMessage) this.next).event.remove();
      } catch (Exception e) { }
      this.removeFirst();
    }
  }

} // END MssMessage
