/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.mss;

import relacs.general.*;
import debug.Debug;

/**
 * The <code>RoutingData</code> class
 *
 * @author  Salvatore Cammarrata
 * @author  Alberto Montresor
 * @since   Jgroup 0.1
 */
final class RoutingData implements MssConstants
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Position constants
  ////////////////////////////////////////////////////////////////////////////////////////////


  private static final int P_LEFTEDGE = HEADER_SIZE;                    // sax
  private static final int P_SWIN = P_LEFTEDGE + Types.DIM_SIZE;      // sax
  private static final int P_LEN  = P_SWIN + SWindow.SWIN_SIZE;         // sax
  private static final int P_DATA = P_LEN + Types.DIM_SIZE;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  int         len;            // Length
  int[]       dest;           // Destinations hosts
  int[]       incarns;        // Incarnation numbers
  int[]       route;          // Routing host
  int[]       delta;          // Distance

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  RoutingData(byte[] buffer)
  {
    int                 pos;            // Read/write position

    len = Types.byte2dim(buffer, P_LEN);
    dest  = new int[len];
    incarns = new int[len];
    route = new int[len];
    delta = new int[len];

    pos = P_DATA;
    for (int i=0; i < len; i++) {
      dest[i] = Types.byte2hid(buffer, pos);
      pos += Types.HID_SIZE;
      incarns[i] = Types.byte2inc(buffer, pos);
      pos += Types.INC_SIZE;
      route[i] = Types.byte2hid(buffer, pos);
      pos += Types.HID_SIZE;
      delta[i] = Types.byte2dim(buffer, pos);
      pos += Types.DIM_SIZE;
    }
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  //
  ////////////////////////////////////////////////////////////////////////////////////////////

  static byte[] message(int len, MssHost scan)
  {
    int                 pos;            // Write position
    byte[]              buffer;         // Buffer to send

    buffer = new byte[Types.DIM_SIZE+len*(2*Types.HID_SIZE+Types.INC_SIZE+Types.DIM_SIZE) +   // sax
                        SWindow.SWIN_SIZE + Types.DIM_SIZE + OVERHEAD_SIZE];    // sax

    Types.dim2byte(buffer, P_LEN, len);
    pos = P_DATA;
    while (scan != null) {
      Types.hid2byte(buffer, pos, scan.address);
      pos += Types.HID_SIZE;
      Types.inc2byte(buffer, pos, scan.address);
      pos += Types.INC_SIZE;
      Types.hid2byte(buffer, pos, scan.route.address);
      pos += Types.HID_SIZE;
      Types.dim2byte(buffer, pos, scan.delta);
      pos += Types.DIM_SIZE;
      scan = scan.getNext();
    }
    return buffer;
  }

  void print()
  {
    if (Debug.DEBUG) {
      for (int i=0; i < len; i++) {
        Debug.print(10, " Dest: " + (dest[i] & 0xFF));
        Debug.print(10, " Route: " + (route[i] & 0xFF));
        Debug.print(10, " Delta: " + delta[i]);
        Debug.println(10, "");
      }
    }

  }

} // END RoutingData

