/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.mss;

import java.net.InetAddress;
import java.util.Hashtable;
import java.util.Random;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.server.RemoteStub;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import relacs.simulator.SocketStatus;

/**
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.5
 */
public class    SocketStatusImpl
extends         UnicastRemoteObject
implements      SocketStatus
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Static fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  private static Random     random = new Random();          // Random number gen.


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  private Hashtable           links;          // Connectivity map


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   */
  SocketStatusImpl()
       throws RemoteException
  {
    Registry reg = null;

    links = new Hashtable(101);
    try {
      reg = LocateRegistry.getRegistry(Registry.REGISTRY_PORT+1);
      reg.bind("SocketStatus", this);
    } catch (Exception e) {
      try {
        reg = LocateRegistry.createRegistry(Registry.REGISTRY_PORT+1);
        reg.bind("SocketStatus", this);
      } catch (Exception e1) {
        System.err.println("Partition simulator not available: " + e);
      }
    }
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *
   */
  boolean isReliable(InetAddress address)
  {
    Integer     prob;
    int val;

    prob = (Integer) links.get(address);
    val = (int) (random.nextFloat() * BASE);
    return (prob == null || val < prob.intValue());
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Remote methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Set the probability of a datagram packet to be correctly delivered at
   *  the host identified by <code> inet</code>. <code>prob</code>/100
   *  corresponds to the probability.
   */
  public void setStatus(InetAddress inet, int prob)
       throws RemoteException
  {
    if (prob >= BASE)
      links.remove(inet);
    else
      links.put(inet, new Integer(prob));
  }

}

