/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.mss;

import java.io.IOException;
import java.net.SocketException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

import java.rmi.RemoteException;

import relacs.mss.SocketStatusImpl;
import relacs.mss.SocketStatusImpl_Stub;
import relacs.mss.SocketStatusImpl_Skel;

/**
 * This class represents an unreliable socket for sending and receiving datagram packets.
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.5
 */
public class    UnreliableSocket
extends         DatagramSocket
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  SocketStatusImpl      status;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Constructs an unreliable datagram socket and binds it to any available port
   * on the local host machine.
   *
   * @exception  SocketException  if the socket could not be opened,
   *               or the socket could not bind to the specified local port.
   */
  public UnreliableSocket()
        throws SocketException
  {
    this(0,null);
  }


  /**
   * Constructs a unreliable datagram socket and binds it to the specified port
   * on the local host machine.
   *
   * @param      local   port to use.
   * @exception  SocketException  if the socket could not be opened,
   *             or the socket could not bind to the specified local port.
   */
  public UnreliableSocket(int port)
    throws SocketException
  {
    this(port, null);
  }


  /**
   * Creates a unreliable datagram socket, bound to the specified local
   * address.  The local port must be between 0 and 65535 inclusive.
   * @param port local port to use
   * @param laddr local address to bind
   */
  public UnreliableSocket(int port, InetAddress laddr)
        throws SocketException
  {
    super(port, laddr);

    try {
      status = new SocketStatusImpl();
    } catch (RemoteException e) {
      System.err.println("Partition simulator not available: " + e);
    }
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Socket methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Sends a datagram packet from this socket. The
   * <code>DatagramPacket</code> includes information indicating the
   * data to be sent, its length, the IP address of the remote host,
   * and the port number on the remote host.
   *
   * @param      p   the <code>DatagramPacket</code> to be sent.
   * @exception  IOException  if an I/O error occurs.
   * @see        java.net.DatagramPacket
   */
  public void send(DatagramPacket p)
        throws IOException
  {
    if (status == null || status.isReliable(p.getAddress()))
      super.send(p);
  }

}


