/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.rmi;

import java.rmi.*;
import java.util.*;

import jgroup.*;
import jgroup.multicast.*;


/**
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.9
 */
final class     AsynchAckListener
implements      AckListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  Callback      cb;             // Callback to be notified at the end.
  HashSet       waiting;        // Members from which we are waiting a result or an exception
  MemberId[]    members;        // View components


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  AsynchAckListener(Callback cb)
  {
    this.cb = cb;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Reacts to view change events generated
   */
  public void notifyView(View view)
  {
    members = view.getMembers();
    for (int i=0; i <= members.length; i++)
      waiting.add(members[i]);
    // Do nothing; we don't need to collect results
  }

  /**
   *
   */
  public void ack(MemberId id, int vpos, Object obj)
  {
    waiting.remove(id);
    if (obj instanceof Exception)
      cb.exception(id, (Exception) obj);
    else
      cb.result(id, obj);
  }

  /**
   *
   */
  public void viewChange()
  {
    MemberId[] members = (MemberId[]) waiting.toArray(new MemberId[waiting.size()]);
    for (int i=0; i < members.length; i++)
      cb.exception(members[i], new RemoteException("Member not reachable"));
  }

} // END AsynchAckListener
