/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.rmi;

import java.io.*;
import java.rmi.*;

import jgroup.*;

import relacs.general.*;

/**
 * Interface <code>GroupDispatcher</code> is the remote interface that must be
 * implemented by external
 *
 * @author  Alberto Montresor
 * @since   0.8
 */
public interface  GroupDispatcher
extends           Remote
{

  /**
   *  Handle unicast method invocation (with return value).
   */
  public Object invoke(IID iid, long hash, Object[] args)
    throws Exception;

  /**
   *  Handle multicast method invocation  (with return value).
   */
  public Object invoke(IID iid, long hash, ByteVectorOutputStream msg)
    throws Exception;

  /**
   *  Handle unicast method invocation (without return value).
   *  (This version is more efficient then the corresponding method with
   *  a return value)
   */
  public void vinvoke(IID iid, long hash, Object[] args)
    throws Exception;

  /**
   *  Handle multicast method invocation  (without return value).
   *  (This version is more efficient then the corresponding method with
   *  a return value)
   */
  public void vinvoke(IID iid, long hash, ByteVectorOutputStream msg)
    throws Exception;

} // END GroupDispatcher
