/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.rmi;

import java.net.*;
import jgroup.*;

/**
 *
 * @author      Alberto Montresor
 * @since       Jgroup 0.8
 */
public class    IIDFactory
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  private int       counter;
  private int       ack;
  private IIDList   issued;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  public IIDFactory()
  {
    counter = 0;
    ack = 0;
    issued = new IIDList();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *
   */
  public synchronized IID getId()
  {
    return new IIDImpl(++counter, ack);
  }

  /**
   *
   */
  public synchronized void ackId(IID id)
  {
    issued.insert(id.getCounter());
    ack = issued.getMaxConsecutive();
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Inner class
  ////////////////////////////////////////////////////////////////////////////////////////////

  final class     IIDList
  {

    private int                   start;          // First element
    private int                   stop;           // Last element
    private IIDList               next;           // Next item in list

    ////////////////////////////////////////////////////////////////////////////////////////////
    // Constructors
    ////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Build an empty <code> IIDList</code>.
     */
    IIDList()
    {
    }

    IIDList(int key, IIDList next)
    {
      this.start = key;
      this.stop  = key;
      this.next  = next;
    }


    ////////////////////////////////////////////////////////////////////////////////////////////
    // Methods
    ////////////////////////////////////////////////////////////////////////////////////////////

    int getMaxConsecutive()
    {
      if (next != null)
        return next.stop;
      else
        return 0;
    }



    synchronized void insert(int key)
    {
      IIDList     scan;           // Used to scan the list
      IIDList     item;           // Temporary variable

      scan = this;

      // Scan the list
      while (scan.next != null && key > scan.next.stop + 1) 
        scan = scan.next;
      if (scan.next == null || key < scan.next.start - 1) {
        // Append at the beginning
        scan.next = new IIDList(key, scan.next);
      } else if (key == scan.next.stop + 1) {
        // Increment the stop position
        item = scan.next.next;
        if (item != null && item.start == key + 1) {
          // Merge the next item
          scan.next.stop = item.stop;
          scan.next.next = item.next;
        } else {
          // Just increment
          scan.next.stop = key;
        }
      } else if (key > scan.next.stop + 1) {
        // Append at the end
        scan.next.next = new IIDList(key, scan.next.next);
      } else if (key == scan.next.start - 1) {
        // Decrement start
        scan.next.start = key;
      }
    }

  } // END IIDList

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Test
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] args)
  {
    IIDFactory factory = new IIDFactory();
    for (int i=1; i < 10; i ++)
      factory.ackId(i);
  }


  public void ackId(int i)
  {
    issued.insert(i);
    ack = issued.getMaxConsecutive();
  }
  
} // END IIDFactory



