/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.rmi;

import java.io.*;

import jgroup.*;


public final class  IIDImpl
implements          IID, Externalizable
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  private VMIDImpl  vmid;
  private int       counter;
  private int       ack;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

  IIDImpl(int counter, int ack)
  {
    this.vmid = VMIDImpl.getLocalVMID();
    this.counter = counter;
    this.ack = ack;
  }

  /**
   *  Default public constructor for externalization.
   */
  public IIDImpl()
  {
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // IID Methods
  ////////////////////////////////////////////////////////////////////////////////////////////


  /**
   *  Returns the <code>VMID</code> of the virtual machine that
   *  generated this invocation.
   */
  public VMID getVmid()
  {
    return vmid;
  }


  /**
   *  Returns the sequential counter identifying this particular
   *  invocation.
   */
  public int getCounter()
  {
    return counter;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Package methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns the sequential counter identifying this particular
   *  invocation.
   */
  int getAck()
  {
    return ack;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Object methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Returns a string representation of the object.
   */
  public String toString()
  {
    return "["+vmid.toString()+","+counter+"]";
  }

  /**
   * Returns a hashcode for the <code>IID</code>.  Two <code>IID</code>s
   * will have the same hashcode if they are equal with respect to their
   * content.
   *
   * @return the hashcode
   */
  public int hashCode() {
    return vmid.hashCode() + (int) counter;
  }

  /**
   * Compares two <code>IID</code>s for content equality.
   *
   * @param obj the Object to compare with
   * @return  true if these Objects are equal; false otherwise.
   */
  public boolean equals(Object obj) {
    if ((obj != null) && (obj instanceof IIDImpl)) {
      IIDImpl iid = (IIDImpl)obj;
      return (vmid.equals(iid.vmid) && counter == iid.counter);
    } else {
      return false;
    }
  }

  /**
   *  Write the content of this IID on stream <tt>out</tt>.
   *  Note that method writeExternal of vmid is called, to
   *  improve the efficiency of this method.
   */
  public void writeExternal(ObjectOutput out)
    throws IOException
  {
    out.writeInt(ack);
    out.writeInt(counter);
    vmid.writeExternal(out);
  }

  /**
   *  Read the content of this IID from stream <tt>in</tt>.
   */
  public void readExternal(ObjectInput in)
    throws IOException, ClassNotFoundException
  {
    ack = in.readInt();
    counter = in.readInt();
    vmid = new VMIDImpl();
    vmid.readExternal(in);
  }


} // END IIDImpl



