/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.rmi;

import java.io.*;
import java.rmi.*;

/**
 *
 * @author      Alberto Montresor
 * @since       Jgroup 0.8
 */
public class      MarshalInputStream
extends           ObjectInputStream
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructors
  ////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Provide a way for subclasses that are completely reimplementing
     * MarshalInputStream to not have to allocate private data just used by
     * this implementation of ObjectInputStream.
     *
     * <p>If there is a security manager installed, this method first calls the
     * security manager's <code>checkPermission</code> method with a
     * <code>SerializablePermission("enableSubclassImplementation")</code>
     * permission to ensure it's ok to enable subclassing.
     *
     * @exception IOException   Thrown if not called by a subclass.
     *
     * @throws SecurityException
     *    if a security manager exists and its
     *    <code>checkPermission</code> method denies
     *    enabling subclassing.
     *
     */
  protected MarshalInputStream()
    throws IOException
  {
    super();
    enableResolveObject(true);
  }

  /**
   *
   */
  public MarshalInputStream(InputStream out)
    throws IOException
  {
    super(out);
    enableResolveObject(true);
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  protected Object resolveObject(Object obj)
    throws IOException
  {
    try {
      return ((MarshalledObject) obj).get();
    } catch (ClassCastException e) {
      return obj;
    } catch (ClassNotFoundException e) {
      throw new MarshalException("resolveObject: ", e);
    }
  }


} // END MarshalInputStream



