/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/*
 * Revision history:
 *
 * 2 Nov. 2000:
 *
 * Added a patch for bug x from Java Developer Connection
 * In Jdk 1.2.1, the serialization of an object implementing Remote 
 * may raise a StubNotFoundException in the following case:
 * - the object class does not extend RemoteStub
 * - the object has not been exported, i.e. it has not called 
 *   UnicastRemoteObject.exportObject
 * This is the case for  _ClientProxy proxy objects. They extend 
 * Remote (actually, they extend the remote interfaces of the remote 
 * object they represent); they do not extend RemoteStub (they do not 
 * carry a single RemoteRef, but a collection of them); they are not 
 * exported (because only remote objects are exported;  proxies are 
 * never exported). 
 *
 * In 1.2.2 and 1.3 things are different. If an object implementing 
 * Remote, not extending RemoteStub and not exported is transmitted, 
 * the system try to serialize the object itself (and not its stub). 
 * This is ok with this code, because my _ClientProxy objects are 
 * externalizable (they serialize the collection of objects).
 *
 * A workaround for Jdk 1.2.1 is to make this class extending class 
 * java.rmi.server.RemoteStub. Please uncomment the "extends"line to 
 * make this class work with 1.2.1.
 *
 * 2 Nov. 2000:
 *   - implements Externalizable instead of implements
 *     Serializable (to solve bug of 1.2.1)
 *   - added default constructor (for externalization)
 *
 * 24 Nov. 2000:
 * Added implementation of interface HandlerGetter, needed for
 * dynamic generation of proxy classes.
 */

package relacs.rmi.proxy;

import java.io.*;


/**
 * Superclass of every group method invocation proxy.
 * 
 *
 * @author Alberto Montresor
 * @since Jgroup 0.9
 */
public class  JgroupProxy
// extends       java.rmi.server.RemoteStub
implements    java.io.Externalizable, HandlerGetter
{

  /**
   * The invocation handler for this proxy instance
   * @serial
   */
  protected JgroupHandler h;

  /**
   * Default constructor
   */
  public JgroupProxy() {
  }

  /**
   * Constructs a new <code>JgroupProxy</code> instance from a subclass
   * (typically, a dynamic proxy class) with the specified
   * invocation handler.
   *
   * @param   h the handler to dispatch method invocations to
   */
  protected JgroupProxy(JgroupHandler h) {
    this.h = h;
  }

  /**
   *  Returns the invocation handler to which invocations are dispatched
   */
  public JgroupHandler getInvocationHandler()
  {
    return h;
  }

  /**
   *  Write the content of this JgroupProxy on stream <tt>out</tt>.
   */
  public void writeExternal(ObjectOutput out)
    throws IOException
  {
    out.writeObject(h);
  }

  /**
   *  Read the content of this JgroupProxy from stream <tt>in</tt>.
   */
  public void readExternal(ObjectInput in)
    throws IOException, ClassNotFoundException
  {
    h = (JgroupHandler) in.readObject();
  }
  


} // END JgroupProxy


