/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.rmi.registry;

import java.rmi.*;
import java.rmi.server.*;
import java.lang.reflect.*;

import relacs.general.*;
import relacs.rmi.*;
import relacs.rmi.proxy.*;


/**
 *
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.5
 */
final class     ServiceData
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** List of interfaces implementented by this object */
  private Class           serverClass;  // List of interfaces implemented by this object

  /** List of bindings */
  private LinkedList      list;         // List of bindings


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  ServiceData(Class serverClass)
  {
    this.serverClass = serverClass;
    list = new LinkedList();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Insert a new binding related to this service. 
   */
  void insert(BindingList binding)
    throws RemoteException
  {
    if (serverClass != null) {
      if (serverClass.equals(binding.serverClass)) {
        list.insert(binding);
      } else {
        throw new RemoteException("Service name " + binding.servicename + 
          " is reserved for members of class " + serverClass.getName());
      }
    }
  }

  /**
   *  Remove a binding from the list.
   */
  void remove(BindingList bl)
  {
    list.remove(bl);
  }

  /**
   *  Returns true if the are no bindings related to this service
   */
  boolean isEmpty()
  {
    return list.isEmpty();
  }


  /**
   *  Return a binding for this service
   */
  Remote getStub()
    throws RemoteException
  {
    LinkedList    scan;
    BindingList   bl;

    ExtGroupHandler2 handler = (ExtGroupHandler2) Exporter.getHandler();
    Remote proxy = Exporter.getProxy(serverClass, handler);

    scan = list.getFirst();
    while (scan != null) {
      bl = (BindingList) scan.getObj();
      handler.insert(bl.remote);
      scan = scan.next();
    }
    return proxy;
  }

} // End ServiceData
