/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package relacs.rmi.registry;

import java.io.*;

import jgroup.*;
import relacs.rmi.*;
import relacs.general.*;

/**
 *
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.4
 */
public final class    Status
implements            Externalizable
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  VMIDImpl[]        vmids;
  IntList[]         removed;
  BindingList[][]   bindings;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Externalization
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void writeExternal(ObjectOutput out)
        throws IOException
  {
    int size;

    // Write vmids
    size=vmids.length;
    out.writeInt(size);
    for (int i=0; i < size; i++)
      vmids[i].writeExternal(out);

    // Write removed
    size=removed.length;
    out.writeInt(size);
    for (int i=0; i < size; i++)
      removed[i].writeExternal(out);

    // Write bindings
    size=bindings.length;
    out.writeInt(size);
    for (int i=0; i < size; i++) {
      int size2 = bindings[i].length;
      out.writeInt(size2);
      for (int j=0; j < size2; j++)
        bindings[i][j].writeExternal(out);
    }
  }

  public void readExternal(ObjectInput in)
        throws IOException, ClassNotFoundException
  {
    int size;

    // Read vmids
    size = in.readInt();
    vmids = new VMIDImpl[size];
    for (int i=0; i < size; i++) {
      vmids[i] = new VMIDImpl();
      vmids[i].readExternal(in);
    }

    // Read removed
    size = in.readInt();
    removed = new IntList[size];
    for (int i=0; i < size; i++) {
      removed[i] = new IntList();
      removed[i].readExternal(in);
    }

    // Read bindings
    size = in.readInt();
    bindings = new BindingList[size][];
    for (int i=0; i < size; i++) {
      int size2 = in.readInt();
      bindings[i] = new BindingList[size2];
      for (int j=0; j < size2; j++) {
        bindings[i][j] = new BindingList();
        bindings[i][j].readExternal(in);
      }
    }
  }

} // END Status
