/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package test.hello;

import jgroup.*;
import jgroup.registry.*;

/**
 * Client program for Hello
 *
 * @author  Alberto Montresor
 * @since   Jgroup 0.1
 */
public class HelloClient
{

  public static void main(String argv[])
    throws Exception
  {
    String hostfile = null;
    int rport = 0;

    for (int i = 0 ; i < argv.length ; i++) {
      if  (argv[i].equals("-rport")) {
        if ((i + 1) == argv.length) {
          System.err.println("Options -rport must be followed by a port number");
          usage();
          return;
        }
        try {
          rport = Integer.parseInt(argv[++i]);
        } catch (Exception e) {
          System.err.println("Value " + argv[i] + " is not a number");
          usage();
          return;
        }
      } else if (argv[i].startsWith("-")) {
        System.err.println("Unknown option " + argv[i]);
        usage();
      } else {
        if (hostfile != null) {
          System.err.println("Hostfile already specified");
          usage();
          return;
        }
        hostfile = argv[i];
      }
    }
    if (hostfile == null) {
      System.err.println("You must specify an hostfile");
      usage();
      return;
    }

    /*
     * Obtain a proxy for the depedable registry running in the
     * distributed system described in the hostfile on the specified port;
     */
    DependableRegistry reg = 
      RegistryFactory.getRegistry(new DistributedSystem(hostfile), rport);

    /*
     * Print the number of groups registered in the dependable registry
     */
    String[] bindings = reg.list();
    System.out.println("Number of elements: " + bindings.length);

    /*
     * Retrieve a proxy for an object group implementing Hello registered under
     * "Jgroup/HelloServer"
     */
    Hello server = (Hello) reg.lookup("Jgroup/HelloServer");

    /*
     * Invoked methods sayHello, which returns an object of type Answer
     * containing a string produced by the contacted object plus
     * the time at which the groups members installed the last view.
     */
    Answer answer = server.sayHello();
    System.out.println(answer.message);
    if (answer.time != null) {
      for (int i=0; i < answer.time.length; i++)
        System.out.print(answer.time[i] + " ");
      System.out.println("");
    }
  }

  private static void usage()
  {
    System.out.println("Usage: helloclient <hostfile> [-rport <port number>]");
  }

} // END HelloClient










