/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package test.jini;

import java.io.*;
import java.net.*;
import java.rmi.server.*;
import java.rmi.*;

import jgroup.*;
import jgroup.jini.*;

import debug.Debug;

import net.jini.core.entry.Entry;
import net.jini.core.lookup.ServiceID;
import net.jini.lookup.ServiceIDListener;
import jgroup.jini.*;


/**
 * Simple Hello server aimed at demonstrating the capabilities of
 * Greg.
 *
 * @author  Alberto Montresor
 * @since   Jgroup 1.0
 */
public class HelloServer
implements Hello, MembershipListener, ServiceIDListener
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** Message returned by sayHello */
  private String answer;

  /** Service proxies */
  private MembershipService   membershipService;
  private ExternalGMIService  externalGMIService;


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor and main
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Instantiates an HelloServer object.
   */
  public static void main(String argv[])
    throws Exception
  {
    String hostfile = null;

    for (int i = 0 ; i < argv.length ; i++) {
      if (argv[i].startsWith("-")) {
        System.err.println("Unknown option " + argv[i]);
        usage();
      } else {
        if (hostfile != null) {
          System.err.println("Hostfile already specified");
          usage();
          return;
        }
        hostfile = argv[i];
      }
    }
    if (hostfile == null) {
      System.err.println("You must specify an hostfile");
      usage();
      return;
    }

    HelloServer obj = new HelloServer("Hello from "+InetAddress.getLocalHost(), hostfile);
  }

  private static void usage()
  {
    System.out.println("Usage: javac test.hello.HelloServer <hostfile>");
  }

  /**
   *  Constructor invoked by main
   */
  public HelloServer(String message, String hostfile)
    throws Exception
  {
    /*
     * Prepares answer message
     */
    answer = message;

    /*
     * Obtains the group manager
     */
    Object[] configuration = new Object[] { new DistributedSystem(hostfile) };
    Object[] listeners = new Object[] { this };
    GroupManager gm = GroupManager.getGroupManager(configuration, listeners);

    /*
     * Obtains the service proxies
     */
    membershipService = (MembershipService) gm.getService(MembershipService.class);
    externalGMIService = (ExternalGMIService) gm.getService(ExternalGMIService.class);

    /*
     * Joins the group
     */
    membershipService.join(12);

    /*
     * Registers itself in the lookup service
     */
    System.setSecurityManager(new RMISecurityManager());
    Entry[] entries = new Entry[] { new GroupEntry("HelloServer") };

    JoinManager joinMgr = new JoinManager(this, entries, this, null, null);
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Hello
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  External method
   */
  public String sayHello()
    throws RemoteException
  {
    Debug.println("sayHello: returning " + answer);
    return answer;
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from MembershipListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void viewChange(View v)
  {
    Debug.println("View id: " + v.getVid());
    MemberId[] members = v.getMembers();
    for (int i=0; i < members.length; i++)
      Debug.println("Member " + i + ": " + members[i]);
  }

  public void prepareChange()
  {
  }

  public void leaved()
  {
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from ServiceIDListener
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void serviceIDNotify(ServiceID serviceID)
  {
    // FIXME Store the service ID somewhere
			System.out.println("HelloServer ServiceID: " + serviceID);
  }

} // END HelloServer

