/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package test.multicast;

import java.io.*;
import java.net.*;

import jgroup.*;
import jgroup.multicast.*;
import debug.Debug;

/**
 * This is a very simple test showing the multicast capabilities
 * of Jgroup (not RMI-based).
 *
 * @author  Alberto Montresor
 * @since   Jgroup 1.0
 */
public class Test1
implements MembershipListener, MulticastListener
{


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Fields
  ////////////////////////////////////////////////////////////////////////////////////////////

  MembershipService   membershipService;
  MulticastService    multicastService;
  MemberId            me;

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  Test1(String hostfile, int jport)
        throws JgroupException, IOException
  {
    Object[] configuration = new Object[] { new DistributedSystem(hostfile),
      new TransportConfig(jport) };
    Object[] listeners = new Object[] { this };
    GroupManager gm = GroupManager.getGroupManager(configuration, listeners);
    membershipService = (MembershipService) gm.getService(MembershipService.class);
    multicastService = (MulticastService) gm.getService(MulticastService.class);
    membershipService.join(7);
    me = membershipService.getMyIdentifier();
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main method
  ////////////////////////////////////////////////////////////////////////////////////////////

  public static void main(String[] argv)
    throws Exception
  {
    String hostfile = null;
    int jport = 0;

    for (int i = 0 ; i < argv.length ; i++) {
      if  (argv[i].equals("-jport")) {
        if ((i + 1) == argv.length) {
          System.err.println("Options -jport must be followed by a port number");
          usage();
          return;
        }
        try {
          jport = Integer.parseInt(argv[++i]);
        } catch (Exception e) {
          System.err.println("Value " + argv[i] + " is not a number");
          usage();
          return;
        }
      } else if (argv[i].startsWith("-")) {
        System.err.println("Unknown option " + argv[i]);
        usage();
      } else {
        if (hostfile != null) {
          System.err.println("Hostfile already specified");
          usage();
          return;
        }
        hostfile = argv[i];
      }
    }
    if (hostfile == null) {
      System.err.println("You must specify an hostfile");
      usage();
      return;
    }

    Test1 t = new Test1(hostfile, jport);
  }

  private static void usage()
  {
    System.out.println("Usage: test1 <hostfile> [-jport <port number>]");
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Upcalls
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void viewChange(View view)
  {
    MemberId[] members = view.getMembers();

    Debug.println("New view identifier: " + view.getVid());
    for (int i=0; i < members.length; i++)
      Debug.println(members[i]);

    try {
      if (me != null) {
        Debug.println("Send msg: " + me);
        multicastService.mcast(me, null);
      }
    } catch (Exception e) {
      Debug.println("Mcast exception: " + e);
      e.printStackTrace();
    }
  }


  public Object deliverStream(InputStream stream)
  {
    Debug.print("Test1: deliverStream: ");
    return null;
  }


  public Object deliverObject(Object obj)
  {
    Debug.println("Test1: deliverObject: " + obj);
    return null;
  }


  public void prepareChange()
  {
  }


  public void leaved()
  {
  }

} // END Test1

