/*
 * Copyright (c) 1998-2000 The Jgroup Team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library Public License for more details.
 *
 * You should have received a copy of the GNU Library Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


package test.performance;

import jgroup.*;

import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;
import java.net.*;
import java.io.*;

/**
 *  RMI server for measuring the performance of standard RMI invocations.
 *  Can be executed remotely through the <t>Executor</t> class,
 *  since it implements the <t>Server</t> interface.
 *
 *  @author     Alberto Montresor
 *  @since        Jgroup 1.1
 */
public class RMIServer 
implements SpeedTest, Server
{

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Main
  ////////////////////////////////////////////////////////////////////////////////////////////

  /** 
   * Start server from command line
   */
  public static void main(String[] argv)
    throws Exception
  {
		int port = Registry.REGISTRY_PORT;
    for (int i = 0 ; i < argv.length ; i++) {
      if  (argv[i].equals("-rport")) {
        if ((i + 1) == argv.length) {
          System.err.println("Options -rport must be followed by a port number");
          usage();
          return;
        }
        try {
          port = Integer.parseInt(argv[++i]);
        } catch (Exception e) {
          System.err.println("Value " + argv[i] + " is not a number");
          usage();
          return;
        }
      } else {
				System.err.println("Unknown option " + argv[i]);
        usage();
      }    
		}
		RMIServer server = new RMIServer();
    Object[] objs = { new Integer(port) };
    server.start(objs);
  }

  private static void usage()
  {
    System.out.println("Usage: rmispeedserver [-rport <port number>]");
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Constructor
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   * Export the server; create a local registry in order
   * to bind the server.
   */
  public RMIServer()
    throws RemoteException, JgroupException, IOException
  {
    UnicastRemoteObject.exportObject(this);
  }

  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from Server
  ////////////////////////////////////////////////////////////////////////////////////////////

  /**
   *  Initialization; nothing to do
   */
  public void start(Object[] args)
		throws Exception
  {
			int port = ((Integer) args[0]).intValue();
    Registry reg = LocateRegistry.createRegistry(port);
    reg.rebind("Jgroup/SpeedTest", this);
    System.out.println("SpeedTest registered");
  }

  /**
   *  Initialization; nothing to do
   */
  public void halt()
    throws Exception
  {
    UnicastRemoteObject.unexportObject(this, false);
  }


  ////////////////////////////////////////////////////////////////////////////////////////////
  // Methods from SpeedTest
  ////////////////////////////////////////////////////////////////////////////////////////////

  public void test() throws RemoteException { }

  public byte[] test(byte[] x) throws RemoteException { return x; }

  public String test(String x) throws RemoteException { return x; }

  public void mtest() throws RemoteException, McastRemoteException 
  { }

  public byte[] mtest(byte[] x) throws RemoteException, McastRemoteException 
  { return x; }

  public String mtest(String x) throws RemoteException, McastRemoteException
  { return x; }

  /**
   *  Shut down the server
   */
  public void shutdown()
    throws RemoteException
  {
    new Thread() {
      public void run() {
        try {
          sleep(3000);
        } catch (InterruptedException e) {
        }
        System.exit(0);
      }
    }.start();
  }

}
